@@ -11,7 +11,6 @@ import electrum
 from electrum.bitcoin import TYPE_ADDRESS
 from electrum import WalletStorage, Wallet
 from electrum_gui.kivy.i18n import _
-from electrum.contacts import Contacts
 from electrum.paymentrequest import InvoiceStore
 from electrum.util import profiler, InvalidPassword
 from electrum.plugins import run_hook
@@ -23,7 +22,7 @@ from kivy.core.window import Window
 from kivy.logger import Logger
 from kivy.utils import platform
 from kivy.properties import (OptionProperty, AliasProperty, ObjectProperty,
-                             StringProperty, ListProperty, BooleanProperty)
+                             StringProperty, ListProperty, BooleanProperty, NumericProperty)
 from kivy.cache import Cache
 from kivy.clock import Clock
 from kivy.factory import Factory
@@ -74,9 +73,60 @@ from electrum.util import base_units
 class ElectrumWindow(App):
 
     electrum_config = ObjectProperty(None)
-
     language = StringProperty('en')
 
+    # properties might be updated by the network
+    num_blocks = NumericProperty(0)
+    num_nodes = NumericProperty(0)
+    server_host = StringProperty('')
+    server_port = StringProperty('')
+    num_chains = NumericProperty(0)
+    blockchain_name = StringProperty('')
+    blockchain_checkpoint = NumericProperty(0)
+
+    auto_connect = BooleanProperty(False)
+    def on_auto_connect(self, instance, x):
+        host, port, protocol, proxy, auto_connect = self.network.get_parameters()
+        self.network.set_parameters(host, port, protocol, proxy, self.auto_connect)
+    def toggle_auto_connect(self, x):
+        self.auto_connect = not self.auto_connect
+
+    def choose_server_dialog(self, popup):
+        from uix.dialogs.choice_dialog import ChoiceDialog
+        protocol = 's'
+        def cb2(host):
+            from electrum.network import DEFAULT_PORTS
+            pp = servers.get(host, DEFAULT_PORTS)
+            port = pp.get(protocol, '')
+            popup.ids.host.text = host
+            popup.ids.port.text = port
+        servers = self.network.get_servers()
+        ChoiceDialog(_('Choose a server'), sorted(servers), popup.ids.host.text, cb2).open()
+
+    def choose_blockchain_dialog(self, dt):
+        from uix.dialogs.choice_dialog import ChoiceDialog
+        chains = self.network.get_blockchains()
+        def cb(name):
+            for index, b in self.network.blockchains.items():
+                if name == self.network.get_blockchain_name(b):
+                    self.network.follow_chain(index)
+                    #self.block
+        names = [self.network.blockchains[b].get_name() for b in chains]
+        if len(names) >1:
+            ChoiceDialog(_('Choose your chain'), names, '', cb).open()
+
+    use_rbf = BooleanProperty(False)
+    def on_use_rbf(self, instance, x):
+        self.electrum_config.set_key('use_rbf', self.use_rbf, True)
+
+    use_change = BooleanProperty(False)
+    def on_use_change(self, instance, x):
+        self.electrum_config.set_key('use_change', self.use_change, True)
+
+    use_unconfirmed = BooleanProperty(False)
+    def on_use_unconfirmed(self, instance, x):
+        self.electrum_config.set_key('confirmed_only', not self.use_unconfirmed, True)
+
     def set_URI(self, uri):
         self.switch_to('send')
         self.send_screen.set_URI(uri)
@@ -91,14 +141,17 @@ class ElectrumWindow(App):
         Logger.info('language: {}'.format(language))
         _.switch_lang(language)
 
+    def update_history(self, *dt):
+        if self.history_screen:
+            self.history_screen.update()
+
     def on_quotes(self, d):
-        #Logger.info("on_quotes")
-        pass
+        Logger.info("on_quotes")
+        self._trigger_update_history()
 
     def on_history(self, d):
-        #Logger.info("on_history")
-        if self.history_screen:
-            Clock.schedule_once(lambda dt: self.history_screen.update())
+        Logger.info("on_history")
+        self._trigger_update_history()
 
     def _get_bu(self):
         return self.electrum_config.get('base_unit', 'mBTC')
@@ -106,17 +159,15 @@ class ElectrumWindow(App):
     def _set_bu(self, value):
         assert value in base_units.keys()
         self.electrum_config.set_key('base_unit', value, True)
-        self.update_status()
-        if self.history_screen:
-            self.history_screen.update()
+        self._trigger_update_status()
+        self._trigger_update_history()
 
     base_unit = AliasProperty(_get_bu, _set_bu)
     status = StringProperty('')
     fiat_unit = StringProperty('')
 
     def on_fiat_unit(self, a, b):
-        if self.history_screen:
-            self.history_screen.update()
+        self._trigger_update_history()
 
     def decimal_point(self):
         return base_units[self.base_unit]
@@ -124,7 +175,7 @@ class ElectrumWindow(App):
     def btc_to_fiat(self, amount_str):
         if not amount_str:
             return ''
-        rate = run_hook('exchange_rate')
+        rate = self.fx.exchange_rate()
         if not rate:
             return ''
         fiat_amount = self.get_amount(amount_str + ' ' + self.base_unit) * rate / pow(10, 8)
@@ -133,7 +184,7 @@ class ElectrumWindow(App):
     def fiat_to_btc(self, fiat_amount):
         if not fiat_amount:
             return ''
-        rate = run_hook('exchange_rate')
+        rate = self.fx.exchange_rate()
         if not rate:
             return ''
         satoshis = int(pow(10,8) * Decimal(fiat_amount) / Decimal(rate))
@@ -193,20 +244,31 @@ class ElectrumWindow(App):
         title = _('Electrum App')
         self.electrum_config = config = kwargs.get('config', None)
         self.language = config.get('language', 'en')
-        self.network = network = kwargs.get('network', None)
-        self.plugins = kwargs.get('plugins', [])
 
+        self.network = network = kwargs.get('network', None)
+        if self.network:
+            self.num_blocks = self.network.get_local_height()
+            self.num_nodes = len(self.network.get_interfaces())
+            host, port, protocol, proxy_config, auto_connect = self.network.get_parameters()
+            self.server_host = host
+            self.server_port = port
+            self.auto_connect = auto_connect
+            self.proxy_config = proxy_config if proxy_config else {}
+
+        self.plugins = kwargs.get('plugins', [])
         self.gui_object = kwargs.get('gui_object', None)
         self.daemon = self.gui_object.daemon
+        self.fx = self.daemon.fx
 
-        self.contacts = Contacts(self.electrum_config)
-        self.invoices = InvoiceStore(self.electrum_config)
+        self.use_rbf = config.get('use_rbf', False)
+        self.use_change = config.get('use_change', True)
+        self.use_unconfirmed = not config.get('confirmed_only', True)
 
         # create triggers so as to minimize updation a max of 2 times a sec
-        self._trigger_update_wallet =\
-            Clock.create_trigger(self.update_wallet, .5)
-        self._trigger_update_status =\
-            Clock.create_trigger(self.update_status, .5)
+        self._trigger_update_wallet = Clock.create_trigger(self.update_wallet, .5)
+        self._trigger_update_status = Clock.create_trigger(self.update_status, .5)
+        self._trigger_update_history = Clock.create_trigger(self.update_history, .5)
+        self._trigger_update_interfaces = Clock.create_trigger(self.update_interfaces, .5)
         # cached dialogs
         self._settings_dialog = None
         self._password_dialog = None
@@ -215,11 +277,11 @@ class ElectrumWindow(App):
         return os.path.basename(self.wallet.storage.path) if self.wallet else ' '
 
     def on_pr(self, pr):
-        if pr.verify(self.contacts):
-            key = self.invoices.add(pr)
+        if pr.verify(self.wallet.contacts):
+            key = self.wallet.invoices.add(pr)
             if self.invoices_screen:
                 self.invoices_screen.update()
-            status = self.invoices.get_status(key)
+            status = self.wallet.invoices.get_status(key)
             if status == PR_PAID:
                 self.show_error("invoice already paid")
                 self.send_screen.do_clear()
@@ -268,7 +330,7 @@ class ElectrumWindow(App):
 
     def switch_to(self, name):
         s = getattr(self, name + '_screen', None)
-        if self.send_screen is None:
+        if s is None:
             s = self.tabs.ids[name + '_screen']
             s.load_screen()
         panel = self.tabs.ids.panel
@@ -380,12 +442,14 @@ class ElectrumWindow(App):
         win = Window
         win.bind(size=self.on_size, on_keyboard=self.on_keyboard)
         win.bind(on_key_down=self.on_key_down)
-        win.softinput_mode = 'below_target'
+        #win.softinput_mode = 'below_target'
         self.on_size(win, win.size)
         self.init_ui()
         self.load_wallet_by_name(self.electrum_config.get_wallet_path())
         # init plugins
         run_hook('init_kivy', self)
+        # fiat currency
+        self.fiat_unit = self.fx.ccy if self.fx.is_enabled() else ''
         # default tab
         self.switch_to('history')
         # bind intent for bitcoin: URI scheme
@@ -396,12 +460,18 @@ class ElectrumWindow(App):
             mactivity = PythonActivity.mActivity
             self.on_new_intent(mactivity.getIntent())
             activity.bind(on_new_intent=self.on_new_intent)
-
+        # connect callbacks
+        if self.network:
+            interests = ['updated', 'status', 'new_transaction', 'verified', 'interfaces']
+            self.network.register_callback(self.on_network_event, interests)
+            self.network.register_callback(self.on_quotes, ['on_quotes'])
+            self.network.register_callback(self.on_history, ['on_history'])
         # URI passed in config
         uri = self.electrum_config.get('url')
         if uri:
             self.set_URI(uri)
 
+
     def get_wallet_path(self):
         if self.wallet:
             return self.wallet.storage.path
@@ -418,7 +488,7 @@ class ElectrumWindow(App):
     def load_wallet_by_name(self, path):
         if not path:
             return
-        wallet = self.daemon.load_wallet(path)
+        wallet = self.daemon.load_wallet(path, None)
         if wallet:
             if wallet != self.wallet:
                 self.stop_wallet()
@@ -426,7 +496,8 @@ class ElectrumWindow(App):
                 self.on_resume()
         else:
             Logger.debug('Electrum: Wallet not found. Launching install wizard')
-            wizard = Factory.InstallWizard(self.electrum_config, path)
+            storage = WalletStorage(path)
+            wizard = Factory.InstallWizard(self.electrum_config, storage)
             wizard.bind(on_wizard_complete=self.on_wizard_complete)
             action = wizard.storage.get_action()
             wizard.run(action)
@@ -516,19 +587,25 @@ class ElectrumWindow(App):
         self.invoices_screen = None
         self.receive_screen = None
         self.requests_screen = None
-
         self.icon = "icons/electrum.png"
-
-        # connect callbacks
-        if self.network:
-            interests = ['updated', 'status', 'new_transaction', 'verified']
-            self.network.register_callback(self.on_network, interests)
-
         self.tabs = self.root.ids['tabs']
 
-    def on_network(self, event, *args):
-        if event == 'updated':
+    def update_interfaces(self, dt):
+        self.num_nodes = len(self.network.get_interfaces())
+        self.num_chains = len(self.network.get_blockchains())
+        chain = self.network.blockchain()
+        self.blockchain_checkpoint = chain.get_checkpoint()
+        self.blockchain_name = chain.get_name()
+        if self.network.interface:
+            self.server_host = self.network.interface.host
+
+    def on_network_event(self, event, *args):
+        Logger.info('network event: '+ event)
+        if event == 'interfaces':
+            self._trigger_update_interfaces()
+        elif event == 'updated':
             self._trigger_update_wallet()
+            self._trigger_update_status()
         elif event == 'status':
             self._trigger_update_status()
         elif event == 'new_transaction':
@@ -548,6 +625,7 @@ class ElectrumWindow(App):
         run_hook('load_wallet', wallet, self)
 
     def update_status(self, *dt):
+        self.num_blocks = self.network.get_local_height()
         if not self.wallet:
             self.status = _("No Wallet")
             return
@@ -565,12 +643,14 @@ class ElectrumWindow(App):
                 text = self.format_amount(c+x+u)
                 status = str(text.strip() + ' ' + self.base_unit)
         else:
-            status = _("Not connected")
+            status = _("Disconnected")
+
         n = self.wallet.basename()
-        self.status = '[size=15dp]%s[/size]\n%s' %(n, status) if n !='default_wallet' else status
+        self.status = '[size=15dp]%s[/size]\n%s' %(n, status)
+        #fiat_balance = self.fx.format_amount_and_units(c+u+x) or ''
 
     def get_max_amount(self):
-        inputs = self.wallet.get_spendable_coins(None)
+        inputs = self.wallet.get_spendable_coins(None, self.electrum_config)
         addr = str(self.send_screen.screen.address) or self.wallet.dummy_address()
         outputs = [(TYPE_ADDRESS, addr, '!')]
         tx = self.wallet.make_unsigned_transaction(inputs, outputs, self.electrum_config)
@@ -719,12 +799,17 @@ class ElectrumWindow(App):
         Clock.schedule_once(lambda dt: on_complete(ok, txid))
 
     def broadcast(self, tx, pr=None):
-        def on_complete(ok, txid):
-            self.show_info(txid)
-            if ok and pr:
-                pr.set_paid(tx.hash())
-                self.invoices.save()
-                self.update_tab('invoices')
+        def on_complete(ok, msg):
+            if ok:
+                self.show_info(_('Payment sent.'))
+                if self.send_screen:
+                    self.send_screen.do_clear()
+                if pr:
+                    self.wallet.invoices.set_paid(pr, tx.txid())
+                    self.wallet.invoices.save()
+                    self.update_tab('invoices')
+            else:
+                self.show_error(msg)
 
         if self.network and self.network.is_connected():
             self.show_info(_('Sending'))
@@ -756,7 +841,7 @@ class ElectrumWindow(App):
         if self.wallet.has_password():
             self.password_dialog(msg, f, args)
         else:
-            apply(f, args + (None,))
+            f(*(args + (None,)))
 
     def delete_wallet(self):
         from uix.dialogs.question import Question
@@ -833,7 +918,7 @@ class ElectrumWindow(App):
 
     def password_dialog(self, msg, f, args):
         def callback(pw):
-            Clock.schedule_once(lambda x: apply(f, args + (pw,)), 0.1)
+            Clock.schedule_once(lambda _: f(*(args + (pw,))), 0.1)
         if self._password_dialog is None:
             from uix.dialogs.password_dialog import PasswordDialog
             self._password_dialog = PasswordDialog()
